(function ($) {

  /**
   * Drupal behaviors.
   */
  Drupal.behaviors.carousel = {
    attach: function (context, settings) {
      if (context.tagName == 'SCRIPT' || !$.fn.slick) {
        return;
      }

      once('carousel', '.carousel', context).forEach(function (element) {
        var $element = $(element);
        var presetName = null;

        // Get preset name
        $.each($element.attr('class').split(' '), function (index, className) {
          var presetMatches = className.match(/^carousel-preset-(.+)$/);
          if (presetMatches) {
            presetName = $.camelCase(presetMatches[1]);
            return false;
          }
        });

        Drupal.carousel($element, presetName);
      });
    }
  };

  Drupal.carousel = function (target, presetName, options) {
    var $element = $(target);

    // Error handlers
    if (!presetName) {
      console.error('Carousel preset name not found.');
      return false;
    }
    if (!Drupal.carouselPresets) {
      console.error('Carousel presets not exist.');
      return false;
    }
    if (!Drupal.carouselPresets[presetName]) {
      console.error('Carousel preset "' + presetName + '" not exist.');
      return false;
    }
    if ($element.length == 0) {
      return false;
    }

    if (!$element.hasClass('carousel')) {
      $element.addClass('carousel');
    }

    options = $.extend({}, Drupal.carouselPresets[presetName], options);

    var $slides = options.slide ? $element.children(options.slide) : $element.children();
    if ($slides.length == 0) {
      $element.addClass('carousel--no-slides');
      return false;
    }
    if ($slides.length == 1) {
      if (options.initEvenWithSingleSlide) {
        $element.addClass('carousel--one-slide');
      }
      else {
        return false;
      }
    }

    // Move arrows and dots to .slick-navigation/.slick-arrows
    var $slickArrows = $element.find('.slick-arrows');
    var $slickNavigation = $element.find('.slick-navigation');
    if (!options.appendArrows) {
      if ($slickArrows.length) {
        options.appendArrows = $slickArrows;
      }
      else if ($slickNavigation.length) {
        options.appendArrows = $slickNavigation;
      }
    }
    if (!options.appendDots && $slickNavigation.length) {
      options.appendDots = $slickNavigation;
    }

    $element.slick(options);

    if (options.vertical) {
      // Fix vertical slick height - remove last visible slide margin from list height
      /** @see Slick.setDimensions() */
      $element.on('setPosition', function () {
        var $firstSlide = $element.find('.slick-slide:first');
        var firstSlideHeight = $firstSlide.outerHeight();
        var firstSlideVerticalMargin = $firstSlide.outerHeight(true) - firstSlideHeight;
        $element.find('.slick-list').height(firstSlideHeight * options.slidesToShow + firstSlideVerticalMargin * (options.slidesToShow - 1));
      });

      // Fix vertical slick height
      /*$element.on('beforeChange', function (event, slick, currentSlideIndex, nextFirstSlideIndex) {
        var $nextFirstSlide = slick.$slides.eq(nextFirstSlideIndex);
        var nextLastSlideIndex = Math.min(nextFirstSlideIndex + slick.options.slidesToScroll, slick.$slides.length - 1);
        var $nextLastSlide = slick.$slides.eq(nextLastSlideIndex);

        var nextFirstSlidePositionTop = $nextFirstSlide.position().top;
        var nextLastSlidePositionTop = $nextLastSlide.position().top;
        var nextLastSlidePositionBottom = nextLastSlidePositionTop + $nextLastSlide.outerHeight();
        var nextSlidesHeight = nextLastSlidePositionBottom - nextFirstSlidePositionTop;

        slick.$list.height(nextSlidesHeight);
      });

      $element.on('setPosition', function (event, slick) {
        var $activeSlides = $element.find('.slick-slide.slick-active');
        var activeSlidesPositionTop = $activeSlides.first().position().top;
        var $lastActiveSlide = $activeSlides.last();
        var activeSlidesPositionBottom = $lastActiveSlide.position().top + $lastActiveSlide.outerHeight();

        slick.$list.height(activeSlidesPositionBottom - activeSlidesPositionTop);
      });*/
    }
  }

})(jQuery);
